json = require("json")

local Deployment = {}

-- This list can be modified to present a prebuild selection of possible automation scripts to start
scripts = {}
	scripts[1] = "standard.ttsx"
	scripts[2] = "StandardScript2.ttsx"
	scripts[3] = "StandardScript3.ttsx"

-- Informations to mount the image source network drive
remoteIpAdress = "" -- The IP-Adress of the remote pc, where the remoteNetworkPath is located. For example: 192.168.253.1
remoteNetworkPath = "" -- The remote networkshare. This name is set in Windows. For example: MyWindowsShare
remoteUsername = "" -- Please set your own of leave it blank, if you setup windows to use passwordless network shares. For example: Volker
remotePassword = "" -- Please set your own of leave it blank, if you setup windows to use passwordless network shares.  For example: Denkhaus
localMountPoint = "/home/partimag" -- This is the default folder and should not be changed.

-- PC- and Tester-Information comment positions. If you change them here, you have to change them also in the clonezilll commandline strings.
targetDriveTesterInfoPos = tsTestInfo.comment19 -- Set the target drive information on comment 19 in the PC- and Tester-Info
sourceImageTesterInfoPos = tsTestInfo.comment20 -- Set the source image information on comment 20 in the PC- and Tester-Info

-- Set the cpations for the two automatic fields. Comment out or change, if you want. Optional, not absolutely required.
tsTestInfo.Set(tsTestInfo.caption19, "Target drive for deployment:")
tsTestInfo.Set(tsTestInfo.caption20, "Source image for deployment:")

-- The selected device identifier for the drive like sda, etc.
selectedDrive = ""

-- The selected image from the network folder to restore on {drive}
selectedImage = ""

-- Determinate in which environment we are. Currently only unix like systems are supported
function determinateOsEnv()
	if package.config:sub(1,1) == "\\" then
		return "Windows"
	elseif package.config:sub(1,1) == "/" then
		return "Unix"
	else
		return "Unknown"
	end
end

-- General: Mounts a network folder in windows or linux environment based on the provided informations
-- Linux: localMountPoint hast to be something like /my-mount-folder (the folder gets created automatically)
-- Windows: localMountPoint hast to be a dirve letter like B or Z or Y
-- status: returns -1 if something has gone wrong or the os specific return code
function mountNetworkShare(remoteIpAdress, remoteNetworkPath, remoteUsername, remotePassword, localMountPoint)
	errorStatus = -1
	if determinateOsEnv() == "Unix" then
		os.execute ("mkdir "..localMountPoint)
		return os.execute ("mount //"..remoteIpAdress.."/"..remoteNetworkPath.." "..localMountPoint.." -o username="..remoteUsername.." -o password="..remotePassword)
	else
		return errorStatus;
	end
	return errorStatus;
end

-- Install clonezilla from the local drive
function installClonezilla()
	errorStatus = -1
	tsApp.StatusLine ("unziping clonezilla from clonezilla.zip in /testlx ...", 0)
	if os.execute ("unzip clonezilla.zip -d /tmp/clonezilla") ~= -1 then
		tsApp.StatusLine ("Installing clonezilla ...", 0)
		if os.execute ("dpkg -i /tmp/clonezilla/*.deb") ~= -1 then
			tsApp.StatusLine ("Successfully and completly installed clonezilla", 0)
		else
			tsApp.StatusLine ("Could not install clonezilla!", 1)
			return errorStatus
		end
	end
	return errorStatus
end

function createFileList(targetPath)
	local fileList = {} -- Empty table for the file list
	if determinateOsEnv()=="Unix" then
		local handle = io.popen("ls -t -d "..targetPath.."/*")
		local result = handle:read("*a")
		handle:close()
		for file in string.gmatch(result, "[^\r\n]+") do
			fileList[#fileList+1] = file -- Create an Index starting by 1 and add the file name to it
		end
	end
	return fileList
end

function searchDirForSystemImage(directory, system)
	tsApp.StatusLine ("Scanning directory "..directory.." for system image "..system, 0)
	local pfile = assert(io.popen(("find '%s' -mindepth 1 -maxdepth 1 -name '"..system.."*' -type d -printf '%%f\\0'"):format(directory), 'r'))
	local list = pfile:read('*a')
	pfile:close()

	local folders = {}

	for filename in string.gmatch(list, '[^%z]+') do
		table.insert(folders, filename)
	end
	table.sort(folders)
	return folders
end

function selectSystemImageDir(selectedIndex, selectedDir, system)
	return searchDirForSystemImage(selectedDir, system)[selectedIndex]
end

-- Creates a list of drive names with name, interface and isSSD to display in select boxes
function getListOfDriveNames()
	local drives = {}
	for index, drive in pairs(tsSystem.GetDrives(tsSystem.drvHarddisk)) do
		local driveName = tsSystem.GetDriveInfo (drive).name ..' - '.. tsSystem.GetDriveInfo (drive).interface
		if tsSystem.GetDriveInfo (drive).isSSD == 1 then
			driveName = driveName .. '-SSD'
		else
			driveName = driveName .. '-HDD'
		end
		drives[#drives+1] = driveName
	end
	return drives
end

-- If we only have one drive and it is a ssd, we directly return this drive
function onlyOneSsdDrive()
	local drives = {}
	for index, drive in pairs(tsSystem.GetDrives(tsSystem.drvHarddisk)) do
		drives[#drives+1] = tsSystem.GetDriveInfo (drive)
	end
	if #drives == 1 and drives[1].isSSD == 1 then
		return drives[1]
	end
	return nil
end

-- If we only have one drive, return it
function onlyOneDrive()
	local drives = {}
	for index, drive in pairs(tsSystem.GetDrives(tsSystem.drvHarddisk)) do
		drives[#drives+1] = tsSystem.GetDriveInfo (drive)
	end
	if #drives == 1 then
		return drives[1]
	end
	return nil
end

function getDriveFromSelection(selectedIndex)
	local localSelection = {}
	for i, drive in pairs(tsSystem.GetDrives(tsSystem.drvHarddisk)) do
		if i == index then
			localSelection = tsSystem.GetDriveInfo (drive)
		end
	end
	return localSelection
end

function showSelectDriveDialog()
	if onlyOneDrive() then
		tsApp.StatusLine ("Automatically selected target disk: "..onlyOneDrive().name, 0)
		selectedDrive = onlyOneDrive()
	elseif onlyOneSsdDrive() then
		tsApp.StatusLine ("Automatically selected target disk: "..onlyOneDrive().name, 0)
		selectedDrive = onlyOneSsdDrive()
	else
		button, index = tsMessageBox.Select ("Select disk as destination", "Please select one of the data carriers on which the Clonezilla image should be imported.", "Disk:", tsMessageBox.Ok, getListOfDriveNames())
		selectedDrive = getDriveFromSelection(index)
		tsApp.StatusLine ("Manually selected target disk: "..selectedDrive.name, 0)
	end
	tsTestInfo.Set (targetDriveTesterInfoPos, minimilizeDeviceName(selectedDrive.devName))
end

function showSelectSourceImage()
	images = searchDirForSystemImage("/home/partimag", getStringWithoutSpaces(tsSystem.GetDMI(tsSystem.dmiSystemName)))
	if #images == 1 then
		selectedImage = selectSystemImageDir(1, "/home/partimag", getStringWithoutSpaces(tsSystem.GetDMI(tsSystem.dmiSystemName)))
	else
		button, index = tsMessageBox.Select ("Select image for deployment", "<b>System information:</b><br>System name: "..tsSystem.GetDMI(tsSystem.dmiSystemName).."<br>Board name: "..tsSystem.GetDMI(tsSystem.dmiBoardName).."< br>BIOS name: "..tsSystem.GetDMI(tsSystem.dmiBiosName).."<br>System UUID: "..tsSystem.GetDMI(tsSystem.dmiSystemUuid), "Image:", tsMessageBox.Ok, searchDirForSystemImage("/home/partimag", getStringWithoutSpaces(tsSystem.GetDMI(tsSystem.dmiSystemName))))
		selectedImage = selectSystemImageDir(index, "/home/partimag", getStringWithoutSpaces(tsSystem.GetDMI(tsSystem.dmiSystemName)))
	end
	tsApp.StatusLine ("Ausgewähltes Quell-Image "..selectedImage.." wird auf die Datenträger "..selectedDrive.name.." zurückgespielt.", 0)
	tsTestInfo.Set (sourceImageTesterInfoPos, selectedImage)
end

-- Show a selection dialog for premade and selected automation scripts
function showAutomationScriptSelection()
	button, index = tsMessageBox.Select ("Start automation script?", "Which of the following automation scripts should be started?", "Start this: ", tsMessageBox.Ok, scripts)
	return scripts[index]
end

function Deployment.execute()
	-- General notification, that the deployment system is setup and working
	tsApp.StatusLine ("toolstar®-Deployment-System is starting ...", 0)
	
	-- Mounting the image source network share
	networkShareMountResult = -1
	if determinateOsEnv() == 'Unix' then
		networkShareMountResult = mountNetworkShare(remoteIpAdress, remoteNetworkPath, remoteUsername, remotePassword, localMountPoint)
		if networkShareMountResult == -1 then
			tsApp.StatusLine ("Could not mount remote network path. Please check the provided informations.", 1)
		end
	end
	
	-- Install clonezilla after mounting the network share
	if networkShareMountResult ~= -1 then
		installClonezilla() -- Prints own error messages. Just call it, after a successfully mount command
	end
	
	-- Display the drive select dialog
	showSelectDriveDialog()
	
	-- Display the source image selection dialog
	showSelectSourceImage()
	
	-- Run the user selected automation script. Just uncomment for a selection dialog.
	-- tsApp.RunScript(showAutomationScriptSelection(), tsApp.runNormal)
	
	-- Automatically start the this automation script. Just change the name.
	-- tsApp.RunScript("standard.ttsx", tsApp.runNormal)
end

-- General utility method
function getStringWithoutSpaces(targetString)
	return targetString:gsub("%s+", "")
end

-- Workaround, because lx dont likes /-lashes in %-Functions, so we just take the important part at the end of the descriptor.
function minimilizeDeviceName(deviceName)
	index = string.find(deviceName, "/[^/]*$")
	return string.sub(deviceName,index+1,string.len(deviceName))
end

return Deployment